#include <cstdio>
#include <cassert>

#include <vector>
#include <iostream>

/**
 * Read a file into a vector.
 *
 * @param filename the filename to read, relative to the cwd
 * @return the vector containing the ints, or an empty vector on error
 */
std::vector<int> readNumbers(const char* filename) {
	assert( filename != nullptr );

	// open the file for reading
	auto fp = std::fopen( filename, "r");
	if ( !fp ) {
		std::perror("opening numbers failed");
		return {};
	}

	std::vector<int> numbers;
	int nCount = 0;

	// read the numbers in (first line = number of inputs)
	std::fscanf( fp, "%d", &nCount);
	for (int i=0; i<nCount; ++i) {
		int nextNum = 0;
		std::fscanf( fp, "%d", &nextNum);
		numbers.push_back( nextNum );
	}

	return numbers;
}

/**
 * Calculate the average value of a vector of ints.
 *
 * @param numbers the numbers to calculate
 * @return the average of the input values
 */
float calcAverage(std::vector<int> numbers) {
	int total = 0;
	unsigned int size = 0; //probably should be size_t

	while ( !numbers.empty() ) {
		total += numbers[0];
		numbers.erase( numbers.begin() );
		size++;
	}

	return total / (float)size;
}

/**
 * Calculate the average value of a vector of ints.
 *
 * This version avoids modifying the vector, so doesn't have the overheads associated with that. It's here as a
 * 'reasonable' approach to the problem. Assuming you can't calculate the sum as you go and avoid the need for
 * the vector all-together...
 *
 * @param numbers the numbers to calculate
 * @return the average of the input values
 */
float calcAverageFast(const std::vector<int>& numbers) {
	int total = 0;

	for( auto& number : numbers ) {
		total += number;
	}

	return total / (float)numbers.size();
}

int main(int argc, const char* argv[]) {
	// check if the correct arguments were provided
	if (argc != 2) {
		std::cerr << "usage: " << argv[0] << " [filename]" << std::endl;
		return EXIT_FAILURE;
	}

	// read the numbers using the above function
	auto numbers = readNumbers(argv[1]);
	if ( numbers.empty() ) {
		std::cerr << "no numbers were read from the file" << std::endl;
		return EXIT_FAILURE;
	}

	// calculate and print the average
	auto avg = calcAverage(numbers);
	std::cout << "the average is: " << avg << std::endl;

	return EXIT_SUCCESS;
}
